#!/bin/sh

#-----------------------------------------------------------------------------------
# Firmware install script for CTM200-R2
# (c) Cypress Solutions Inc.
#-----------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------
#
# Modify the following variables depending on the type of fw upgrade package
# 
# a) for uImage/ipk install (incremental upgrade) adjust and uncomment next 2 lines
INSTALLTYPE=incremental
# list of packages to install 
CTMFW_PACKAGE="ctm200-fw_*_am335x_evm.ipk"
CTMDOC_PACKAGE="ctm200-doc_*_am335x_evm.ipk"
LIBCTM_PACKAGE="libctm1_*_am335x_evm.ipk"
LIBCYPRESS_PACKAGE="libcypress1_*_am335x_evm.ipk"
LIBJSMN_PACKAGE="libjsmn0_*_armv7ahf-vfp-neon-3.2.ipk"
LIBLMDB_PACKAGE="liblmdb0_*_armv7ahf-vfp-neon-3.2.ipk"
LIBJSONCPP_PACKAGE="libjsoncpp1_*_armv7ahf-vfp-neon-3.2.ipk"
CCMCLIENT_PACKAGE="ccm-client_*_am335x_evm.ipk"
MINICOM_PACKAGE="minicom_*_armv7ahf-vfp-neon-3.2.ipk"
VSFTPD_PACKAGE="vsftpd_*_armv7ahf-vfp-neon-3.2.ipk"
ATFTP_PACKAGE="atftpd_*_armv7ahf-vfp-neon-3.2.ipk"
DROPBEAR_PACKAGE="dropbear_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE="lighttpd_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_DIRLISTING="lighttpd-module-dirlisting_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_ACCESS="lighttpd-module-access_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_ACCESSLOG="lighttpd-module-accesslog_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_AUTH="lighttpd-module-auth_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_STATICFILE="lighttpd-module-staticfile_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_INDEXFILE="lighttpd-module-indexfile_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_CGI="lighttpd-module-cgi_*_armv7ahf-vfp-neon-3.2.ipk"
LIGHTTPD_PACKAGE_REDIRECT="lighttpd-module-redirect_*_armv7ahf-vfp-neon-3.2.ipk"

LIBCRYPTO_PACKAGE="libcrypto1.0.0_*_armv7ahf-vfp-neon-3.2.ipk"
LIBSSL_PACKAGE="libssl1.0.0_*_armv7ahf-vfp-neon-3.2.ipk"
OPENSSL_PACKAGE="openssl_*_armv7ahf-vfp-neon-3.2.ipk"

CURL_PACKAGE="curl_*_armv7ahf-vfp-neon-3.2.ipk"
LIBCURL_PACKAGE="libcurl5_*_armv7ahf-vfp-neon-3.2.ipk"
LIBSSH2_PACKAGE="libssh2-1_*_armv7ahf-vfp-neon-3.2.ipk"

BASH_PACKAGE="bash_*_armv7ahf-vfp-neon-3.2.ipk"
ZIP_PACKAGE="zip_*_armv7ahf-vfp-neon-3.2.ipk"

COOVA_CHILLI_PACKAGE="coova-chilli_*_armv7ahf-vfp-neon-3.2.ipk"
OPENVPN_PACKAGE="openvpn_*_armv7ahf-vfp-neon-3.2.ipk"

NCURSESW_PACKAGE="libncursesw5_*_armv7ahf-vfp-neon-3.2.ipk"
LIBPYTHON_PACKAGE="libpython2.7-1.0_*_armv7ahf-vfp-neon-3.2.ipk"
PYTHON_CORE_PACKAGE="python-core_*_armv7ahf-vfp-neon-3.2.ipk"
PYTHON_LANG_PACKAGE="python-lang_*_armv7ahf-vfp-neon-3.2.ipk"
PYTHON_RE_PACKAGE="python-re_*_armv7ahf-vfp-neon-3.2.ipk"
PYTHON_READLINE_PACKAGE="python-readline_*_armv7ahf-vfp-neon-3.2.ipk"

CONNTRACK_TOOLS_PACKAGE="conntrack-tools_*_armv7ahf-vfp-neon-3.2.ipk"
LIBMNL_PACKAGE="libmnl0_*_armv7ahf-vfp-neon-3.2.ipk"
LIBNETFILTER_CONNTRACK_PACKAGE="libnetfilter-conntrack3_*_armv7ahf-vfp-neon-3.2.ipk"
LIBNETFILTER_CTHELPER_PACKAGE="libnetfilter-cthelper0_*_armv7ahf-vfp-neon-3.2.ipk"
LIBNETFILTER_CTTIMEOUT_PACKAGE="libnetfilter-cttimeout1_*_armv7ahf-vfp-neon-3.2.ipk"
LIBNETFILTER_QUEUE_PACKAGE="libnetfilter-queue1_*_armv7ahf-vfp-neon-3.2.ipk"
LIBNFNETLINK_PACKAGE="libnfnetlink0_*_armv7ahf-vfp-neon-3.2.ipk"

# packages to be removed
THTTPD_PACKAGE="thttpd"

# 
# b) for full rootfs install uncomment the next line
#INSTALLTYPE=full
#
#-----------------------------------------------------------------------------------

# The following flag file will be used by the bootloader to determine which
# partition to boot 
ROOTFSFLAG=/var/data/ctm/ROOTFS2


install_incremental ()
{
    # backup rootfs
    echo "Backing up rootfs $croot to $broot"
    /bin/cat $croot > $broot
    if [ "$?" != "0" ]; then
        echo "failed to create backup - exiting"
        return -1
    fi

    # set next boot to backup rootfs 
    if [ "$crootnum" -eq 2 ];then
        echo '1' > $ROOTFSFLAG 
    else
        /bin/rm -f $ROOTFSFLAG >/dev/null
    fi

    # install fw
    echo "Installing updated firmware files"    
    /bin/mount -o remount,rw /dev/root /
    # remove thttpd
    /usr/bin/opkg remove --force-remove --force-depends $THTTPD_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$CTMDOC_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$CTMFW_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBCTM_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBCYPRESS_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBJSMN_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBLMDB_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBJSONCPP_PACKAGE
    /usr/bin/opkg install --force-downgrade --force-overwrite /tmp/.upgrade/$CCMCLIENT_PACKAGE
    # if there are more packages to be installed, add them here
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$ATFTP_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$DROPBEAR_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$MINICOM_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$VSFTPD_PACKAGE
    # lighttpd packages :
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_DIRLISTING
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_ACCESS
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_STATICFILE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_INDEXFILE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_ACCESSLOG
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_AUTH
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_CGI
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE_REDIRECT
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIGHTTPD_PACKAGE
    # openssl/libssl/libcrypto
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBSSL_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBCRYPTO_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$OPENSSL_PACKAGE
    # libssh2/libcurl/curl
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBSSH2_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$LIBCURL_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$CURL_PACKAGE
    # bash
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$BASH_PACKAGE
    # zip 
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$ZIP_PACKAGE
    # coova chilli captive portal
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$COOVA_CHILLI_PACKAGE
    # openvpn
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$OPENVPN_PACKAGE
    # python
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$NCURSESW_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBPYTHON_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$PYTHON_CORE_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$PYTHON_LANG_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$PYTHON_RE_PACKAGE
    /usr/bin/opkg install --force-downgrade --nodeps /tmp/.upgrade/$PYTHON_READLINE_PACKAGE
    # conntrack
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBNFNETLINK_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBNETFILTER_QUEUE_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBNETFILTER_CTTIMEOUT_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBNETFILTER_CTHELPER_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBNETFILTER_CONNTRACK_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$LIBMNL_PACKAGE
    /usr/bin/opkg install --force-downgrade /tmp/.upgrade/$CONNTRACK_TOOLS_PACKAGE

    #######################################################################
    # base-files from rootfs
    /bin/cp -f /tmp/.upgrade/preserve /etc
    /bin/cp -f /tmp/.upgrade/CYPRESS-MIB.txt /etc
    /bin/cp -f /tmp/.upgrade/pcyp.json /etc

    # install the new kernel image
    /bin/cp -f /tmp/.upgrade/uImage /boot

    # install kernel drivers
    /bin/cp -f /tmp/.upgrade/cfg80211.ko /lib/modules/3.2.0/updates/net/wireless
    /bin/cp -f /tmp/.upgrade/mac80211.ko /lib/modules/3.2.0/updates/net/mac80211
    /bin/cp -f /tmp/.upgrade/wl12xx.ko /lib/modules/3.2.0/updates/drivers/net/wireless/wl12xx
    /bin/cp -f /tmp/.upgrade/wl12xx_sdio.ko /lib/modules/3.2.0/updates/drivers/net/wireless/wl12xx
    /bin/cp -f /tmp/.upgrade/wl12xx_spi.ko /lib/modules/3.2.0/updates/drivers/net/wireless/wl12xx

    # reset next boot to current rootfs 
    if [ "$crootnum" -eq 2 ];then
        /bin/rm -f $ROOTFSFLAG >/dev/null
    else
        echo '1' > $ROOTFSFLAG 
    fi
    return 0
}


install_full ()
{
    # formate alternate partition
    /sbin/mkfs.ext4 $broot
    if [ "$?" != "0" ]; then
        echo "Failed format new image partition - exiting"
        return -1
    fi

    # mount mirror partition
    /bin/mkdir -p /var/volatile/mntsd
    /bin/mount $broot /var/volatile/mntsd
    if [ "$?" != "0" ]; then
        echo "Failed mount new image partition - exiting"
        return -1
    fi

    # unpack file system
    echo "Unpacking new firmware image"    
    /bin/tar -zxf /tmp/.upgrade/nfs_root.tar.gz -C /var/volatile/mntsd 
    if [ "$?" != "0" ]; then
        echo "Failed to unpack image - exiting"
        return -1
    fi

    /bin/umount $broot
    /bin/rmdir /var/volatile/mntsd
 
    # set next boot to backup rootfs 
    if [ "$crootnum" -eq 2 ];then
        echo '1' > $ROOTFSFLAG 
    else
        /bin/rm -f $ROOTFSFLAG >/dev/null
    fi
    return 0
}

#-----------------------------------------------------------------------------------
# The following function should be called by the firmware upgrade script
#-----------------------------------------------------------------------------------

ctm200_upgrade_fw ()
{
    # create a /var/data/ctm/config.sh prior to upgrade
    echo "<PRE>"
    echo "Preparing configuration prior to install"
    echo "#!/bin/sh" > /var/volatile/ctm/config
    /usr/bin/cmd showconfig >> /var/volatile/ctm/config
    /bin/cp /var/volatile/ctm/config /var/data/ctm/config.sh
	
    # need to re-enter the following commands as they are auto-configured by other commands (e.g. can prot/usbprot, aux mode)
    echo "# Values that are auto-configured" >> /var/data/ctm/config.sh
    /usr/bin/cmd can baud | /bin/grep can >> /var/data/ctm/config.sh
    /usr/bin/cmd can std | /bin/grep can >> /var/data/ctm/config.sh
    /usr/bin/cmd aux lead | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux leadterm | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux len | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux2 lead | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux2 leadterm | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux2 len | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux3 lead | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux3 leadterm | /bin/grep aux >> /var/data/ctm/config.sh
    /usr/bin/cmd aux3 len | /bin/grep aux >> /var/data/ctm/config.sh

    # manually save APN
    /usr/bin/cmd apn | /bin/grep apn >> /var/data/ctm/config.sh
    # manually save TCPPERSIST
    /usr/bin/cmd tcppersist | /bin/grep tcppersist >> /var/data/ctm/config.sh

    # migrate legacy rmtssh/rmtweb/rmttelnet
    if [ ! -e /usr/bin/cmds/access ]; then
        rmt=$(/usr/bin/cmd rmttelnet|/bin/grep rmttelnet| /usr/bin/awk '{print $3}')
        if [ "$rmt" = "1" ]; then
            port=$(/usr/bin/cmd telnetport| /bin/grep telnetport| /usr/bin/awk '{print $3}')
            /usr/bin/cmd access cell telnetport $port | /bin/grep telnetport >> /var/data/ctm/config.sh
        else
            echo 'cmd access cell telnetport 0' >> /var/data/ctm/config.sh
        fi

        rmt=$(/usr/bin/cmd rmtssh|/bin/grep rmtssh| /usr/bin/awk '{print $3}')
        if [ "$rmt" = "1" ]; then
            port=$(/usr/bin/cmd sshport| /bin/grep sshport| /usr/bin/awk '{print $3}')
            /usr/bin/cmd access cell sshport $port | /bin/grep telnetport >> /var/data/ctm/config.sh
        else
            echo 'cmd access cell sshport 0' >> /var/data/ctm/config.sh
        fi

        rmt=$(/usr/bin/cmd rmtweb|/bin/grep rmtweb| /usr/bin/awk '{print $3}')
        if [ "$rmt" = "1" ]; then
            port=$(/usr/bin/cmd webport|/bin/grep webport| /usr/bin/awk '{print $3}')
            /usr/bin/cmd access cell webport $port | /bin/grep telnetport >> /var/data/ctm/config.sh
        else
            echo 'cmd access cell webport 0' >> /var/data/ctm/config.sh
        fi
    fi

    /usr/bin/dos2unix /var/data/ctm/config.sh
    /usr/sbin/dos2unix /var/data/ctm/config.sh

    # migrate custom firewalls (if any)
    if [ -e /var/config/firewall.custom ] ; then
        /bin/cp -f /var/config/firewall.custom /var/data
    fi

    # migrate auxpollstr.txt, aux2pollstr.txt, aux3pollstr.txt
    /bin/cp -f /var/config/aux*pollstr.txt /var/data/ctm

    echo "Installing CTM-200 Firmware"
    croot=$1
    broot=$2
    if [ "$INSTALLTYPE" = "incremental" ] ; then
        install_incremental
        ret=$?
    fi
    if [ "$INSTALLTYPE" = "full" ] ; then
        install_full
        ret=$? 
    fi

    echo "Cleaning up"
    echo "</PRE>"
    /bin/mount -o remount,ro /dev/root /
    /bin/rm -rf /tmp/.upgrade
    return $ret
}
	
